/*******************************************************************************
 *	BdsMetadata.cc	BDS MetaData Managment Utility
 *				T.Barnaby,	BEAM Ltd,	2011-06-07
 *******************************************************************************
 *
 * This program provides a command line ability to modify BDS MetaData
 *
 */
#include <iostream>
#include <iomanip>
#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <stdarg.h>
#include <errno.h>
#include <BdsD.h>
#include <BdsC.h>
#include <BdsLib.h>
#include <BTable.h>
#include <BDuration.h>
#include <Main.h>
#include <Process.h>
#include <clex.cpp>
#include <cgram.h>
#include <BDebug.h>

using namespace std;
using namespace Bds;

Node*	result;
BString	fileName;

static void print(FILE* file, int tab, const char* fmt, ...){
	va_list	ap;
	
	va_start(ap, fmt);
	while(tab--)
		fprintf(file, "  ");
	vfprintf(file, fmt, ap);
}

void produceTree(Node* n, FILE* file, int tab){
	BIter	i;
	
	print(file, tab, "%s %s {\n", Node::nodeTypeName(n->nodeType()).retStr(), n->name().retStr());
	for(n->nodes().start(i); !n->nodes().isEnd(i); n->nodes().next(i)){
		if(n->nodes()[i])
			produceTree(n->nodes()[i], file, tab + 1);
		else {
			print(file, tab + 1, "Null none {\n");
			print(file, tab + 1, "}\n");
		}
	}
	print(file, tab, "}\n");
}

BError execute(AdminAccess& bds, Parameters& params){
	BError			err;
	BError			e;
	int			ret;
	Process			process(bds, params);

	if(err = process.init())
		return err;

	if(params.commandFile != ""){
		if(freopen(params.commandFile, "r", stdin) == NULL){
			fprintf(stderr, "Unable to open: %s\n", params.commandFile.retStr());
			return 1;
		}

		fileName = params.commandFile;
	}
	else {
		yy_scan_string(params.command);
		fileName = "ComandLine";
	}


	// Process the input file
	if(ret = yyparse())
		return 1;

#ifdef ZAP
#ifndef ZAP
	FILE*	fout = fopen("out.txt", "w");
	produceTree(result, fout, 0);
#else
	produceTree(result, stdout, 0);
#endif
#endif

	if(err = bds.transactionStart())
		return err;

	err = process.run(result);

	if(e = bds.transactionEnd(err))
		err = e;
	
	if(err){
		err.set(err.getErrorNo(), BString("Line: ") + process.line() + ": " + err.getString());
	}

	return err;
}

void usage(void) {
	cerr << "Usage:\tbdsMetadata [options] <command> [argument]\n";
	cerr << " -help                 - Help on command line parameters\n";
	cerr << " -v                    - Verbose messages\n";
//	cerr << " -dryRun               - Just prints what would be done. Default operation\n";
	cerr << " -host <hostname>      - BDS Server host name\n";
	cerr << " -user <user:password> - user id and password\n";
	cerr << " -name <server name>	- The BDS server name\n";
	cerr << " -c <command>          - A command to run\n";
	cerr << " -f <commandFile>      - A command file to run\n";
	cerr << " -d <variable=value>   - Set the named variable to the value given\n";
	cerr << " -maxChanges <n>       - The maximum number of objects that can be changed in a command (0 is unlimited, default is 1)\n";
}

static struct option options[] = {
		{ "?",			0, NULL, 0 },
		{ "h",			0, NULL, 0 },
		{ "help",		0, NULL, 0 },
		{ "v",			0, NULL, 0 },
		{ "dryRun",		0, NULL, 0 },
		{ "host",		1, NULL, 0 },
		{ "user",		1, NULL, 0 },
		{ "name",		1, NULL, 0 },
		{ "c",			1, NULL, 0 },
		{ "f",			1, NULL, 0 },
		{ "d",			1, NULL, 0 },
		{ "maxChanges",		1, NULL, 0 },
		{ 0,0,0,0 }
};

int main(int argc, char** argv){
	BError			err;
	int			optIndex = 0;
	int			c;
	BString			s;
	BString			hostName;
	BString			user;
	BString			password;
	Parameters		params;
	AdminAccess		bds;
	BStringList		strList;
	BDictString		info;

	bdebug = 0;

	// Set stdout to line buffering
	setlinebuf(stdout);

	hostName = getenv("BDS_HOST");
	if(hostName == "")
		hostName = "localhost";

	while((c = getopt_long_only(argc, argv, "", options, &optIndex)) == 0){
		s = options[optIndex].name;
		if(s == "help" || s == "h" || s == "?"){
			usage();
			return 1;
		}
		else if(s == "v"){
			params.verbose = 1;
		}
		else if(s == "dryRun"){
			params.dryRun = 1;
		}
		else if(s == "host"){
			hostName = optarg;
		}
		else if(s == "user"){
			s = optarg;
			user = s.pullToken(":");
			password = s;
		}
		else if(s == "name"){
			params.serverName = optarg;
		}
		else if(s == "c"){
			params.command = optarg;
		}
		else if(s == "f"){
			params.commandFile = optarg;
		}
		else if(s == "d"){
			strList = BString(optarg).split('=');
			if(strList.number() != 2){
				cerr << "Varaibles need to be set using varable=value syntax\n";
				usage();
				return 1;
			}
			if(strList[0][0] != '$')
				strList[0] = BString("$") + strList[0];
			params.variables[strList[0]] = strList[1];
		}
		else if(s == "maxChanges"){
			params.maxChanges = atoi(optarg);
		}
		else {
			usage();
			return 1;
		}
	}
	
	if((argc - optind) > 0){
		usage();
		return 1;
	}

	// Check flags and arguments	

	// Open BDS access
	err = bds.connectService(BString("//") + hostName + "/bdsAdminAccess");

	// Connect to service
	if(!err)
		err = bds.connect(user, password);
	
	// Check server name
	if(!err){
		if(!(err = bds.statisticsGet(info))){
			if(info["name"] != params.serverName)
				err.set(1, BString("Servername incorrect. Connected to: ") + info["name"]);
		}
	}
	
	
	if(!err){
		err = execute(bds, params);
	}
	
	if(err)
		cerr << "Error: " << err.getString() << "\n";

	if(err.getErrorNo() < 0)
		return 1;
	else
		return err.getErrorNo();
}
